﻿using System;
using System.Diagnostics;
using System.Text;
using Scheduler;
using Scheduler.Classes;

namespace HSPI_MQTT
{
    public class HSPI : MarshalByRefObject
    {        
        private Plugin m_plugin = Plugin.GetInstance();
        private hsapplication hs;
                
        // Common Interface - All plug-ins must support these methods and properties
        #region	"Common Plug-In Interface Procedures"

        
        // Short name for this plugin
        public string Name()
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::Name");
            
            return Plugin.PLUGIN_NAME;
        }

        public int Capabilities()
        {
            //	OR the capabilities together to let HS know what this plug-in is capable of doing
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::Capabilities");

            return HomeSeer.CA_IO;
        }
        
        public int AccessLevel()
        {
            //	Return the type of licensing that the plug-in uses
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::AccessLevel");

            return HomeSeer.AL_FREE;
        }

        public bool HSCOMPort()
        {
            //	TRUE if plug-in uses COM ports and you want to use HS pages 
            //	for configuration, FALSE otherwise
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::HSCOMPort");

            return false;
        }

        public bool SupportsHS2()
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::SupportsHS2");

            return true;
        }

        public bool SupportsConfigDevice2()
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::SupportsConfigDevice2");

            return false; // true;
        }

        public short InterfaceStatus()
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::InterfaceStatus");

            //	TODO ... get the real interface status (if I have any)
            return HomeSeer.IS_ERR_NONE;
        }

        public void RegisterEvent(int eventType, hsapplication hsapp, object reference) 
        {            
            hsapp.RegisterEventCB(ref eventType, ref reference);
        }

        public void RegisterCallback(ref object frm)
        {
            hs = (hsapplication)((clsHSPI)frm).GetHSIface();
            object me = this;
                       
            // Register events
            //RegisterEvent(HomeSeer.EV_TYPE_X10, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_LOG, hs, me);
            RegisterEvent(HomeSeer.EV_TYPE_STATUS_CHANGE, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_AUDIO, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_X10_TRANSMIT, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_CONFIG_CHANGE, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_STRING_CHANGE, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_SPEAKER_CONNECT, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_CALLER_ID, hs, me);
            //RegisterEvent(HomeSeer.EV_TYPE_ZWAVE, hs, me);            

            //	Set the global information for the HS interfaces here now that we have them
            m_plugin.SetHomeSeerCallback((clsHSPI)frm);
            m_plugin.Initialize();            
        }

        public void HSEvent(int evtype, object[] parms)
        {
            Plugin.TraceEvent(TraceEventType.Information, string.Format("HSEvent type: {0}  param1: {1} param2: {2} param3: {3}", evtype, parms[1], parms[2], parms[3], parms[4]));
        }

        public void HSEvent(object[] parms)
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSEvent");

            switch ((int)parms[0])
            {

                case HomeSeer.EV_TYPE_X10:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("X10 event for {0}{1}: param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_LOG:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("Log event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;

                case HomeSeer.EV_TYPE_STATUS_CHANGE:
                    //Plugin.TraceEvent(TraceEventType.Information, string.Format("Status change {0}{1} Status: {2} Value: {3} | param2: {4} param3: {5}", parms[2], parms[1], parms[3], parms[4], parms[5], parms[6]));
                    
                    string deviceCode = string.Format("{0}{1}", parms[2], parms[1]);
                    int status = (int)parms[3];
                    string value = (string)parms[4];
                    this.m_plugin.PublishStatusChange(deviceCode, status);
                    this.m_plugin.PublishValueChange(deviceCode, value);
                    
                    break;
                case HomeSeer.EV_TYPE_AUDIO:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("Audio event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_X10_TRANSMIT:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("X10 transmit event for {1}{0}: param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_CONFIG_CHANGE:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("Config Change event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_STRING_CHANGE:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("String change event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    
                    deviceCode = (string)parms[1];
                    value = (string)parms[2];
                    this.m_plugin.PublishStringChange(deviceCode, value);                    
                    break;

                case HomeSeer.EV_TYPE_SPEAKER_CONNECT:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("Speaker connect event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_CALLER_ID:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("CallerID event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
                case HomeSeer.EV_TYPE_ZWAVE:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("ZWave event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;        
                default:
                    Plugin.TraceEvent(TraceEventType.Information, string.Format("* Unknown/undefined event for {0} to: '{1}' | param2: {2} param3: {3}", parms[1], parms[2], parms[3], parms[4]));
                    break;
            }            
        }
               
        public string InitIO(ref int port)
        {
            string message =string.Empty;

            try
            {
                m_plugin.SetupMQTTCallbacks();
                m_plugin.RegisterSubscriptions();

                Plugin.TraceEvent(TraceEventType.Information, "Successfully initialized");
            }
            catch
            {
                message = "Error occured during initialization";
            }
            
            return message;
        }

        public void ShutdownIO()
        {
            Plugin.TraceEvent(TraceEventType.Information, "Shutting down");

            m_plugin.DisconnectFromBroker();
        }

        public void SetIOEx(object dv, string houseCode, string deviceCode, Int16 command, Int16 brightness, Int16 data1, Int16 data2, string voiceCommand, string host) 
        {
            //Plugin.TraceEvent(TraceEventType.Verbose, "HSPI::SetIOEx");
        }               
        
        #endregion 

        //Public Function GenPage(Optional ByRef lnk As String = "") As String
        public string GenPage(string link = "")
        {
            return "<h1>Oh my god!</h1>";
        }
       
    }
}
