﻿using System;
using System.Configuration;
using System.Collections;
using System.Diagnostics;
using System.Text;
using Scheduler;
using Scheduler.Classes;
using System.Collections.Generic;
using MqttLib;
using System.IO;
using System.Xml.Serialization;


namespace HSPI_MQTT
{   
    class Plugin
    {
        //	Public string used for devices to identify them as belonging to this plug-in
        public const string PLUGIN_NAME = "MQTT"; // "HSPI_TEST";

        public const string PLUGIN_DEVICE_TYPE = "MQTT Topic";
                
        //	Config file m_sUnitName
        private const string CFG_FILE = @"config\hspi_mqtt.config";
        private const string CFG_FILE_TST = @"config\hspi_mqtt.xml";

        //	Configuration keys
        //public const string CFG_KEY_ISZONED = "envZoned";
        public const string CFG_KEY_LOGLEVEL = "logLevel";
        //public const string CFG_KEY_MAXUNITS = "numUnits";
        //public const string CFG_KEY_UNITPORT = "UnitPort_";
        //public const string CFG_KEY_MSGDELAY = "msgDelay";
        //public const string CFG_KEY_MSGTIMEOUT = "msgTimeOut";
        //public const string CFG_KEY_MSGWAITACK = "msgWaitAck";
        //public const string CFG_KEY_BAUDRATE = "msgBaudRate";

        //	Configuration values
        public const string CFG_LOGLEVEL_NONE = "0";
        public const string CFG_LOGLEVEL_CRITICAL = "1";
        public const string CFG_LOGLEVEL_ERROR = "2";
        public const string CFG_LOGLEVEL_WARNING = "3";
        public const string CFG_LOGLEVEL_INFO = "4";
        public const string CFG_LOGLEVEL_VERBOSE = "5";
        
        //	Private constants
        //private const int HVAC_MAXIMUM_DEVCODE = 99;
        private const string LOG_PREFIX = "mqtt_plugin_messages_";
        //private const string HVAC_MASTER_LOCATION = "HVAC";
        //private const string HVAC_MASTER_TYPE = "Enviracom Master";
        //private const string HVAC_MASTER_NAME = "Enviracom Master Device";
        //private const string HVAC_MASTER_MAGIC = "Enviracom Master Device";
        //private const string HVAC_MASTER_DEVCODE = "99";

        private const int PERIODIC_TICK_TIME = 1; // in seconds

        static Plugin s_objSingletonInstance = null;
        static readonly object s_objLock = new object();

        private TraceEventType m_eTraceLevel = TraceEventType.Verbose; // TraceEventType.Critical;

        private Configuration m_cfgApp = null;	        // Configuration access (simple object)
        private TraceSource m_trcApp = null;	        // For diagnostic tracing (simple object)
        private clsHSPI m_ifHSPI = null;	            // Interface to HomeSeer HSPI
        private hsapplication hsApp = null;	        // Interface to HomeSeer Application

        private string m_mqttServer = string.Empty;
        private string m_mqttClientId = string.Empty;
        
        public Settings m_settings;        
        public MappingList m_mappings;

        private IMqtt _client;
                
        //	Accessor method to get one and only instance of this Singleton object
		static public Plugin GetInstance()
		{
            if (s_objSingletonInstance == null)
                lock (s_objLock)
			    {
                    if (s_objSingletonInstance == null)
                    {
					    s_objSingletonInstance = new Plugin();
				    }
                }

            return s_objSingletonInstance;
        }

        public hsapplication ifHSApp
        {
            get { return hsApp; }
        }

        
        /// <summary>
        /// 
        /// </summary>
        public void Initialize()
        {

            ////hs.WaitSec( 10 )

            //	Now that we have the HomeSeer object, open up the configuration
            InitConfiguration();                                  

            //	Register the config link here
            object cfg = new ConfigPage();
         //   hsApp.RegisterConfigLink(ref cfg, PLUGIN_NAME);
         //   hsApp.RegisterLinkEx(ref cfg, PLUGIN_NAME);


            //	Init our tracing capability				
            //InitTrace();
            //InitUnits();

            //	Trace this event after we init the trace (otherwise it won't show up)
            //TraceEvent(TraceEventType.Verbose, "DataLogger::Initialize");           

            //hsApp.WriteLog(PLUGIN_NAME, "Successfully initialized");            
        }

        public void SetHomeSeerCallback(clsHSPI objHSPI)
        {
            //	Assign locally 
            m_ifHSPI = objHSPI;
            hsApp = (hsapplication)m_ifHSPI.GetHSIface();
        }

        private Settings ReadSettings()
        {
            string settingsFile = hsApp.GetAppPath() + "\\" + CFG_FILE;
            Settings settings;

            using (var reader = new StreamReader(settingsFile))
            {
                XmlSerializer deserializer = new XmlSerializer(typeof(Settings), new XmlRootAttribute("Settings"));
                settings = (Settings)deserializer.Deserialize(reader);
            }

            if (settings.HomeSeerTopic == null) settings.HomeSeerTopic = "homeseer";

            return settings;
        }

        private void WriteSettings()
        {            
            string settingsFile = hsApp.GetAppPath() + "\\" + CFG_FILE;

            // e.g. my test to create a file
            using (var writer = new FileStream(settingsFile, FileMode.Create))
            {
                //XmlSerializer ser = new XmlSerializer(typeof(List<User>),  new XmlRootAttribute("user_list"));
                XmlSerializer ser = new XmlSerializer(typeof(Settings), new XmlRootAttribute("Settings"));

                Settings settings = new Settings();
                settings.MQTTServer = "tcp://192.168.2.10:1883";

                //Mapping item = new Mapping();
                //item.DeviceId = "A10"; item.Topic.Name = "smartmeter/consumed_low";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A11"; item.Topic.Name = "smartmeter/consumed_high";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A12"; item.Topic.Name = "smartmeter/actual_rate";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A13"; item.Topic.Name = "smartmeter/actual_consumed";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A14"; item.Topic.Name = "smartmeter/gas";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A15"; item.Topic.Name = "smartmeter/actual_produced";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A16"; item.Topic.Name = "smartmeter/produced_low";
                //settings.Mappings.Items.Add(item);
                //item = new Mapping();
                //item.DeviceId = "A17"; item.Topic.Name = "smartmeter/produced_high";
                //settings.Mappings.Items.Add(item);                                
                
                ser.Serialize(writer, settings);
            }
        }

        private void InitConfiguration()
        {

            try
            {
                TraceEvent(TraceEventType.Information, "Reading configuration settings");

                //WriteSettings();
                m_settings = ReadSettings();

                m_mqttServer = m_settings.MQTTServer;
                m_mqttClientId = string.Format("{0}{1}", hsApp.ApplicationName(), DateTime.UtcNow.Ticks);
                if (m_settings.MQTTClientId != null)
                {
                    m_mqttClientId = m_settings.MQTTClientId;
                }

                m_mappings = m_settings.Mappings;
            }
            catch
            {
                TraceEvent(TraceEventType.Error, "Error occured reading configuration");
            }
            
        }


        public void SetupMQTTCallbacks()
        {
            TraceEvent(TraceEventType.Information, string.Format("Connecting to broker {0}", m_mqttServer));  

            // Instantiate client using MqttClientFactory
            _client = MqttClientFactory.CreateClient(m_mqttServer, m_mqttClientId);

            // Setup some useful client delegate callbacks
            _client.Connected += new ConnectionDelegate(_client_Connected);
            _client.ConnectionLost += new ConnectionDelegate(_client_ConnectionLost);
            _client.PublishArrived += new PublishArrivedDelegate(_client_PublishArrived);

            _client.Connect(true);
        }

        public void DisconnectFromBroker()
        {
            if (_client.IsConnected)
            {                
                _client.Disconnect();
                TraceEvent(TraceEventType.Information, "Disconnected from broker");                
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public void RegisterSubscriptions()
        {
            Int16 index;
            DeviceClass device;

            TraceEvent(TraceEventType.Information, "Registering subscriptions");

            foreach (Mapping map in m_mappings.Items)
            {
                index = hsApp.DeviceExists(map.DeviceId);
                device = hsApp.GetDevice(index);

                if (index == -1)
                {
                    TraceEvent(TraceEventType.Information, string.Format("HomeSeer device {0} not found, creating new", map.DeviceId));
    
                    device = hsApp.NewDeviceEx(map.Topics.Items[0].Name);
                    device.hc = map.DeviceId.Substring(0, 1);
                    device.dc = map.DeviceId.Substring(1);
                    device.@interface = PLUGIN_NAME;                    
                    device.location = "MQTT";
                    device.status = HomeSeer.Stat_Unknown;
                    device.misc = HomeSeer.MISC_STATUS_ONLY; // &HomeSeer.MISC_SETVALUE_NOTIFY;
                    device.iotype = HomeSeer.IOTYPE_VARIABLE; // HomeSeer.IOTYPE_INPUT;
                    device.iomisc = map.Topics.Items[0].Name;                    
                    device.dev_type_string = PLUGIN_DEVICE_TYPE;
                }
                else
                {
                    map.DeviceReference = device.@ref;
                    map.DeviceName = device.Name;
                    device.iomisc = map.Topics.Items[0].Name;
                    if (device.Name == "UnKnown") { device.Name = map.Topics.Items[0].Name; }                    
                }
                
                //_client.Subscribe(map.Topic.Name, QoS.BestEfforts);                
                _client.Subscribe(map.Topics.Items[0].Name, QoS.BestEfforts);                
            }

            //DeviceClass d = hsApp.NewDeviceEx("TEST");
            
            
        }

        public MappingList LookupMappingsByTopic(string topicName) 
        {
            MappingList matches = new MappingList();

            foreach (Mapping map in m_mappings.Items)
            {
                if (map.Topics.Items.Count > 0)
                {
                    for (int i = 0; i < map.Topics.Items.Count; i++)
                    {
                        if (map.Topics.Items[i].Name == topicName)
                        {
                            matches.Items.Add(map);
                        }
                    }
                }

                //foreach (Topic topic in (IEnumerable<Topic>)map.Topics)
                //{
                //    if (topic.Name == topicName)
                //    {
                //        matches.Items.Add(map);
                //    }
                //}   
            }

            return matches;
        }

        //public Mapping LookupMappingByTopic(string topic)
        //{
        //    foreach (Mapping map in m_mappings.Items)
        //    {
        //        if (map.Topic.Name == topic) { return map; }
        //    }
        //    return null;
        //}

        public string LookupDeviceCodeByTopic(string topicName)
        {
            foreach (Mapping map in m_mappings.Items)
            {
                if (map.Topics.Items.Count>0)
                {                
                    for (int i = 0; i < map.Topics.Items.Count; i++)
                    {
                        if (map.Topics.Items[i].Name == topicName)
                        {
                            return map.DeviceId;
                        }
                    }
                }

                //foreach (Topic topic in (IEnumerable<Topic>)map.Topics)
                //{
                //    if (topic.Name == topicName)
                //    {
                //        return map.DeviceId;
                //    }
                //}
            }
            return string.Empty;
        }

        public DeviceClass FindDeviceByTopic(string topic)
        {
            DeviceClass device = null;

            string deviceCode = LookupDeviceCodeByTopic(topic);
            if (deviceCode != string.Empty)
            {
                device = LookupDeviceByDeviceCode(deviceCode);
            }
            return device;
        }

        public DeviceClass LookupDeviceByDeviceCode(string deviceCode)
        {
            int index = hsApp.DeviceExists(deviceCode);
            return hsApp.GetDevice(index);                
        }        

        public int LookupDeviceRefByDeviceCode(string deviceCode)
        {
            Scheduler.clsDeviceEnumeration devices = (Scheduler.clsDeviceEnumeration)hsApp.GetDeviceEnumerator();
            Scheduler.Classes.DeviceClass device;
            while (!devices.Finished)
            {
                device = devices.GetNext();
                if ((device.hc + device.dc) == deviceCode)
                {
                    return device.@ref;
                }                
            }
            return -1;
        }

      

        void _client_Connected(object sender, EventArgs e)
        {
            TraceEvent(TraceEventType.Information, "Connected to broker");            
            //RegisterOurSubscriptions();
            //PublishSomething();
        }

        void _client_ConnectionLost(object sender, EventArgs e)
        {
            TraceEvent(TraceEventType.Information, "Broker connecting lost");            
        }

        void RegisterOurSubscriptions()
        {
            //Console.WriteLine("Subscribing to mqttdotnet/subtest/#\n");
            //_client.Subscribe("mqttdotnet/subtest/#", QoS.BestEfforts);
        }

        void PublishSomething()
        {
            //Console.WriteLine("Publishing on mqttdotnet/pubtest\n");
            //_client.Publish("mqttdotnet/pubtest", "Hello MQTT World", QoS.BestEfforts, false);
        }

        public void PublishStatusChange(string deviceCode, int status)
        {            
            int deviceRef = LookupDeviceRefByDeviceCode(deviceCode);
            if (deviceRef > 0)
            {
                string topic = string.Format("{0}/{1}/status", m_settings.HomeSeerTopic, deviceRef);
                MqttPayload payload = status.ToString();
                _client.Publish(topic, payload, QoS.BestEfforts, false);
            }
        }

        public void PublishValueChange(string deviceCode, string value)
        {
            int deviceRef = LookupDeviceRefByDeviceCode(deviceCode);
            if (deviceRef > 0)
            {
                string topic = string.Format("{0}/{1}/value", m_settings.HomeSeerTopic, deviceRef);
                MqttPayload payload = value;
                _client.Publish(topic, payload, QoS.BestEfforts, false);
            }
        }

        public void PublishStringChange(string deviceCode, string value)
        {
            int deviceRef = LookupDeviceRefByDeviceCode(deviceCode);
            if (deviceRef > 0)
            {
                string topic = string.Format("{0}/{1}/string", m_settings.HomeSeerTopic, deviceRef);
                MqttPayload payload = value;
                _client.Publish(topic, payload, QoS.BestEfforts, false);
            }
        }

        bool _client_PublishArrived(object sender, PublishArrivedArgs e)
        {
            Console.WriteLine("Received Message");
            Console.WriteLine("Topic: " + e.Topic);
            Console.WriteLine("Payload: " + e.Payload);            
            Console.WriteLine();

            string deviceCode = LookupDeviceCodeByTopic(e.Topic);
            DeviceClass device = FindDeviceByTopic(e.Topic);


            //Mapping map = LookupMappingByTopic(e.Topic);
            MappingList matches = LookupMappingsByTopic(e.Topic);
            foreach (Mapping map in (IEnumerable<Mapping>)matches)
            {                          
                if (device != null)
                {
                    if (IsNumeric(e.Payload))
                    {
                        string s = e.Payload.ToString(); //.Replace('.', ',');

                        decimal j;
                        bool result = decimal.TryParse(s, out j);
                    
                        int n = Convert.ToInt32(j);

                        //decimal n = Convert.ToInt32(float.Parse(s) * 1000);

                        if (map.Format != null)
                        {                        
                            s = String.Format(map.Format, e.Payload);
                            hsApp.SetDeviceString(deviceCode, s);
                        }
                        else
                        {
                            hsApp.SetDeviceString(deviceCode, s);
                        }
                        hsApp.SetDeviceValue(deviceCode, n);
                    }
                    else
                    {
                        hsApp.SetDeviceString(deviceCode, e.Payload);
                    }
                    hsApp.SetDeviceLastChange(deviceCode, DateTime.Now);

                    // Check and update device name (into topic name) if neccesary                
                    if (device.Name == "UnKnown") { device.Name = e.Topic; }                   
                
                }
                else
                {
                    TraceEvent(TraceEventType.Warning, string.Format("HomeSeer device {0} no longer found", deviceCode));            
                }

            }

            //if (deviceCode != string.Empty) 
            //{
            //    if (IsNumeric(e.Payload))
            //    {
            //        string s = e.Payload.ToString().Replace('.', ',');
            //        int n = Convert.ToInt32(float.Parse(s) * 1000);
                    
            //        hsApp.SetDeviceValue(deviceCode, n);
            //        hsApp.SetDeviceString(deviceCode, s);
            //    }
            //    else
            //    {
            //        hsApp.SetDeviceString(deviceCode, e.Payload);
            //    }
            //    hsApp.SetDeviceLastChange(deviceCode, DateTime.Now);

            //    // Check and update device name (into topic name) if neccesary                
            //    int deviceRef = LookupDeviceRefByDeviceCode(deviceCode);
            //    if (deviceRef != -1) 
            //    {
            //        Mapping map = LookupMappingByTopic(e.Topic);

            //        DeviceClass device = hsApp.GetDeviceByRef(deviceRef);
            //        if (device.Name == "UnKnown") { device.Name = e.Topic; }

            //        if (map.Format != null) { 
            //            string f = map.Format;
            //            string s = String.Format(f, e.Payload);
            //            hsApp.SetDeviceString(deviceCode, s);
            //        }
            //    }
            //}
            return true;
        }


        #region "Tracing and Diagnostics  "
       
        private void WriteHomeseerLog(TraceEventType eType, string szMessage)
        {
            if (ifHSApp != null)
                ifHSApp.WriteLog(PLUGIN_NAME, string.Format("[{0}] {1}", eType.ToString(), szMessage));
        }

        private void InternalTraceEvent(TraceEventType eType, string sMessage)
        {
            //	Send this to our TraceSource if it is valid
            if (m_trcApp != null)
                m_trcApp.TraceEvent(eType, 0, sMessage);
            if (eType <= m_eTraceLevel)
                WriteHomeseerLog(eType, sMessage);
        }

        private void InternalLogInformation(string sMessage)
        {
            //	Send this to our TraceSource if it is valid
            if (m_trcApp != null)
                m_trcApp.TraceInformation(sMessage);
            WriteHomeseerLog(TraceEventType.Information, sMessage);
        }

        static public void TraceEvent(TraceEventType eType, string sMessage)
        {
            GetInstance().InternalTraceEvent(eType, sMessage);
        }

        static public void LogInformation(string sMessage)
        {
            GetInstance().InternalLogInformation(sMessage);
        }

        public void SetTraceLevel(string sLevel)
        {
            TextWriterTraceListener txt;

            txt = (TextWriterTraceListener)m_trcApp.Listeners[1];
            switch (sLevel)
            {
                case CFG_LOGLEVEL_NONE:
                    m_eTraceLevel = TraceEventType.Critical;
                    break;
                case CFG_LOGLEVEL_CRITICAL:
                    txt.Filter = new EventTypeFilter(SourceLevels.Critical);
                    m_trcApp.Switch.Level = SourceLevels.Critical;
                    m_eTraceLevel = TraceEventType.Critical;
                    break;
                case CFG_LOGLEVEL_ERROR:
                    txt.Filter = new EventTypeFilter(SourceLevels.Error);
                    m_trcApp.Switch.Level = SourceLevels.Error;
                    m_eTraceLevel = TraceEventType.Error;
                    break;
                case CFG_LOGLEVEL_WARNING:
                    txt.Filter = new EventTypeFilter(SourceLevels.Warning);
                    m_trcApp.Switch.Level = SourceLevels.Warning;
                    m_eTraceLevel = TraceEventType.Warning;
                    break;
                case CFG_LOGLEVEL_INFO:
                    txt.Filter = new EventTypeFilter(SourceLevels.Information);
                    m_trcApp.Switch.Level = SourceLevels.Information;
                    m_eTraceLevel = TraceEventType.Information;
                    break;
                case CFG_LOGLEVEL_VERBOSE:
                    txt.Filter = new EventTypeFilter(SourceLevels.Verbose);
                    m_trcApp.Switch.Level = SourceLevels.Verbose;
                    m_eTraceLevel = TraceEventType.Verbose;
                    break;
            }
        }

        public string GetConfig(string sConfig, string sDefault)
        {
            string sRet = null;
            KeyValueConfigurationElement kvElem;

            kvElem = m_cfgApp.AppSettings.Settings[sConfig];
            if (kvElem == null)
                sRet = sDefault;
            else
                sRet = kvElem.Value;

            return sRet;
        }

        public void SetConfig(string sKey, string sVal)
        {
            try
            {
                if (m_cfgApp.AppSettings.Settings[sKey] != null)
                {
                    m_cfgApp.AppSettings.Settings.Remove(
                        sKey);
                }
            }
            catch (Exception)
            {
                //	This happens if it doesn't exist, ignore it
            }
            m_cfgApp.AppSettings.Settings.Add(
                sKey,
                sVal);

            // Save the configuration file.
            m_cfgApp.Save(ConfigurationSaveMode.Modified);

            // Force a reload of a changed section.
            ConfigurationManager.RefreshSection("appSettings");

            //	Check if this was the trace level
            if (sKey == CFG_KEY_LOGLEVEL)
                SetTraceLevel(sVal);
        }

        #endregion

        private static bool IsNumeric(string s)
        {
            float output;
            return float.TryParse(s, out output);
        }
    }       
}
